############################################
## Bivariate Exponential-Exponential CWM  ##
## Binomial, Poisson, Gaussian & Gamma    ##
############################################

source("ML.R")
library("numDeriv")

EECWM <- function(
	Y,			                 # Y: a numerical vector for the response variable
  X,			                 # X: a numerical vector for the unique covariate
  familyX = "Gaussian",      # the exponential distribution used for X 
	familyY = "Gaussian",      # the exponential distribution used for Y|x 
	k=2,                     # number of groups
	mX=1,                    # mX=1 for Bernoulli; mX>1 for Binomial
	mY=1,                    # mY=1 for Bernoulli; mY>1 for Binomial
	method="BFGS",           # optimization method
  initialization="random.soft", # initialization procedure: "random.soft", "random.hard", and "manua" 
	start.z=NULL,                 # (n x k)-matrix of soft or hard classification: it is used only if initialization="manual"		
	iter.max=1000,                # maximum number of iterations in the EM-algorithm
	threshold=1.0e-04,            # stopping rule in the Aitken rule
	loglikplot=TRUE,              # if TRUE, the log-likelihood values against the iterations are plotted
	SE = FALSE,                   # if SE = TRUE, then Standard Errors are computed
	EMevolutionplot=FALSE         # clusters representation versus EM-iterations 
  )
{
  
#call = match.call()

n     <- length(Y)  # sample size
prior <- numeric(k) # weights
Z     <- rep(1,n)   # useful for glm() on X

# X

dispX   <- numeric(k)   # Dispersion for each group 
nuX     <- NULL         # only for the Gamma distribution
VarFunX <- numeric(k)   # Variance Function for each group
VarX    <- numeric(k)   # Variance for each group
muX     <- numeric(k)
PX      <- array(0,c(n,k),dimnames=list(1:n,paste("comp.",1:k,sep="")))

# Y|x

beta0   <- numeric(k)
beta1   <- numeric(k)
dispY   <- numeric(k)   # Conditional Dispersion for each group
nuY     <- NULL         # only for the Gamma distribution   
VarFunY <- array(0,c(n,k),dimnames=list(1:n,paste("comp.",1:k,sep="")))
VarY    <- array(0,c(n,k),dimnames=list(1:n,paste("comp.",1:k,sep="")))
muY     <- array(0,c(n,k),dimnames=list(1:n,paste("comp.",1:k,sep="")))
PY      <- array(0,c(n,k),dimnames=list(1:n,paste("comp.",1:k,sep="")))

# ------------------------- #
# posteriors initialization #
# ------------------------- #

# RANDOM INITIALIZATION

if(initialization=="random.soft"){

	z  <- array(runif(n*k),c(n,k)) # soft posterior probabilities (no-normalized) (n x k) 
	z  <- z/rowSums(z)             # soft posterior probabilities (n x k)

  } 

if(initialization=="random.hard"){
  
  z  <- t(rmultinom(n, size = 1, prob=rep(1/k,k)))  # hard posterior probabilities (n x k)

  } 

if(initialization=="manual"){ # z.start can be both soft and hard initialization
  
  z  <- start.z      # posterior probabilities (n x k) no-normalized

  } 

# ------------ #
# EM algorithm #
# ------------ #

# Preliminary definition of convergence criterions

check     <- 0
iteration <- 1
loglik    <- NULL
aloglik   <- NULL
aloglik   <- c(0,0)
a         <- NULL
a         <- c(0,0)

while(check<1){

	# ++++++ #
	# M-step #
	# ++++++ #

  # ------- #
  # Weights #
  # ------- #

	prior <- colMeans(z)

  # - #
	# X #
	# - #
  
	print("X")
	
	for(h in 1:k){
	  if(familyX=="Gaussian"){
	    modelX      <- GaussianX(X,weights=z[,h])
	    muX[h]      <- modelX$muX
	    dispX[h]    <- modelX$sigmaX^2
	    VarFunX[h]  <- 1     
	    VarX[h]     <- dispX[h]*VarFunX[h]     
	    PX[,h]      <- dnorm(X,mean=muX[h],sd=sqrt(VarX[h]))              
	  }
	}
	for(h in 1:k){
	  if(familyX=="Poisson"){
	    modelX      <- PoissonX(X,weights=z[,h])
	    muX[h]      <- modelX$lambdaX
	    dispX[h]    <- 1
	    VarFunX[h]  <- muX[h]     
	    VarX[h]     <- dispX[h]*VarFunX[h]     
	    PX[,h]      <- dpois(X,lambda=muX[h])              
	  }
	}
	for(h in 1:k){
	  if(familyX=="Binomial"){
	    modelX      <- BinomialX(X,weights=z[,h],m=mX)
	    muX[h]      <- mX*modelX$pX
	    dispX[h]    <- 1/mX
	    VarFunX[h]  <- modelX$pX*(1-modelX$pX)     
	    VarX[h]     <- dispX[h]*VarFunX[h]     
	    PX[,h]      <- dbinom(X,size=mX,prob=modelX$pX)              
	  }
	}
	for(h in 1:k){
	  if(familyX=="Gamma"){
	    modelX      <- GammaX(X,weights=z[,h])
	    muX[h]      <- modelX$muX
	    dispX[h]    <- 1/modelX$nuX
	    VarFunX[h]  <- muX[h]^2     
	    VarX[h]     <- dispX[h]*VarFunX[h]
      nuX[h]      <- modelX$nuX
	    PX[,h]      <- dgam(X, mu = muX[h], nu = nuX[h])              
	  }
	}
  
  # --- #
	# Y|x #
	# --- #
  
  print("Y|x")
  
	for(h in 1:k){
	  if(familyY=="Gaussian"){
      modelY      <- GaussianY(Y,X,weights=z[,h]) #,mustart=muY[,h]
      beta0[h]    <- modelY$beta0
      beta1[h]    <- modelY$beta1
      muY[,h]     <- beta0[h]+beta1[h]*X
      dispY[h]    <- modelY$sigmaY^2
      VarFunY[,h] <- rep(1,n)     
      VarY[,h]    <- dispY[h]*VarFunY[,h]     
      PY[,h]      <- dnorm(Y,mean=muY[,h],sd=sqrt(VarY[,h]))              
	  }
	}
	for(h in 1:k){
	  if(familyY=="Poisson"){
	    modelY      <- PoissonY(Y,X,weights=z[,h]) #,mustart=muY[,h]
	    beta0[h]    <- modelY$beta0
	    beta1[h]    <- modelY$beta1
	    muY[,h]     <- exp(beta0[h]+beta1[h]*X)
	    dispY[h]    <- 1
	    VarFunY[,h] <- muY[,h]     
	    VarY[,h]    <- dispY[h]*VarFunY[,h]     
	    PY[,h]      <- dpois(Y,lambda=muY[,h])              
	  }
	}
	for(h in 1:k){
	  if(familyY=="Binomial"){
	    modelY      <- BinomialY(Y,X,weights=z[,h],m=mY) #,mustart=muY[,h]
	    beta0[h]    <- modelY$beta0
	    beta1[h]    <- modelY$beta1
	    muY[,h]     <- exp(beta0[h]+beta1[h]*X)/(1+exp(beta0[h]+beta1[h]*X))
	    dispY[h]    <- 1/mY
	    VarFunY[,h] <- muY[,h]*(1-muY[,h])     
	    VarY[,h]    <- dispY[h]*VarFunY[,h]     
	    PY[,h]      <- dbinom(Y,size=mY,p=muY[,h])              
	  }
	}
	for(h in 1:k){
	  if(familyY=="Gamma"){
	    modelY      <- GammaY(Y,X,weights=z[,h],method=method) #,mustart=muY[,h]
	    beta0[h]    <- modelY$beta0
	    beta1[h]    <- modelY$beta1
	    muY[,h]     <- exp(beta0[h]+beta1[h]*X)
	    dispY[h]    <- 1/modelY$nuY
	    VarFunY[,h] <- muY[,h]^2     
	    VarY[,h]    <- dispY[h]*VarFunY[,h]     
	    nuY[h]      <- modelY$nuY
      PY[,h]      <- dgam(Y, mu = muY[,h], nu = nuY[h])              
	  }
	}
	
  # ------------------------------------- # 
  # Global - Observed-data log-likelihood # 
  # ------------------------------------- #
  
	llvalues          <- log(rowSums(matrix(rep(prior,n),n,k,byrow=TRUE)*PY*PX)) 
	loglik[iteration] <- sum(llvalues)
  	
  # ----------------------------------------------- #
	# Aitkane's Acceleration-Based Stopping Criterion #
	# ----------------------------------------------- #
	
	if(iteration>2 & k > 1){
	  a[iteration-1]      <- (loglik[iteration]-loglik[iteration-1])/(loglik[iteration-1]-loglik[iteration-2])
	  aloglik[iteration]  <- loglik[iteration-1]+(1/(1-a[iteration-1])*(loglik[iteration]-loglik[iteration-1]))
	  if(abs(aloglik[iteration]-loglik[iteration])<threshold) 
	    check <- 1 
	}
	
	if(iteration==iter.max | k==1) check <- 1
  
  if(EMevolutionplot & k > 1){
  
    gr <- apply(z,1,which.max) 
    
    if(familyY=="Gaussian"){
      
      postscript(paste("prova_",(iteration-1),".eps"), height=8, width=8, horizontal=F, paper="special", onefile=F)  
      par(mai=c(0.82,0.78,0.4,0.024)) 
      par(omi=c(0,0,0,0))
      plot(X,Y,col=gr,cex=0.6,pch=gr,lwd=2) 
      for(h in 1:k){ 
        griglia <- seq(min(X[which(gr==h)]),max(X[which(gr==h)]),length=101)
        points(griglia,beta0[h]+beta1[h]*griglia,lty=h,col=h,type="l")
      }
      
      dev.off()
      
    }
    if(familyY=="Poisson"){
    
      postscript(paste("prova_",(iteration-1),".eps"), height=8, width=8, horizontal=F, paper="special", onefile=F)  
      par(mai=c(0.82,0.78,0.4,0.024)) 
      par(omi=c(0,0,0,0))
      plot(X,Y,col=gr,cex=0.6,pch=gr,lwd=2) 
      for(h in 1:k){ 
        griglia <- seq(min(X[which(gr==h)]),max(X[which(gr==h)]),length=101)
        points(griglia,exp(beta0[h]+beta1[h]*griglia),lty=h,col=h,type="l")
      }
      
      dev.off()
    
    }
    if(familyY=="Binomial"){
      
      postscript(paste("prova_",(iteration-1),".eps"), height=8, width=8, horizontal=F, paper="special", onefile=F)  
      par(mai=c(0.82,0.78,0.4,0.024)) 
      par(omi=c(0,0,0,0))
      plot(X,Y,col=gr,cex=0.6,pch=gr,lwd=2) 
      for(h in 1:k){ 
        griglia <- seq(min(X[which(gr==h)]),max(X[which(gr==h)]),length=101)
        points(griglia,mY*exp(beta0[h]+beta1[h]*griglia)/(1+exp(beta0[h]+beta1[h]*griglia)),lty=h,col=h,type="l")
      }
      
      dev.off()
      
    }
    if(familyY=="Gamma"){
      
      postscript(paste("prova_",(iteration-1),".eps"), height=8, width=8, horizontal=F, paper="special", onefile=F)  
      par(mai=c(0.82,0.78,0.4,0.024)) 
      par(omi=c(0,0,0,0))
      plot(X,Y,col=gr,cex=0.6,pch=gr,lwd=2) 
      for(h in 1:k){ 
        griglia <- seq(min(X[which(gr==h)]),max(X[which(gr==h)]),length=101)
        points(griglia,exp(beta0[h]+beta1[h]*griglia),lty=h,col=h,type="l")
      }
      
      dev.off()
      
    } 
  }
	
	cat("Iteration",iteration,"\n")
	iteration <- iteration + 1
  
	# ++++++ #
	# E-Step #
	# ++++++ #
	
	z.num  <- matrix(rep(prior,n),n,k,byrow=TRUE)*PY*PX  # (n x k)
	z.den  <- rowSums(z.num)                             # n-vector
	z      <- z.num/matrix(rep(z.den,k),ncol=k)          # (n x k)
	
}

finalloglik <- loglik[iteration-1] 
	
# ----------------------------------------------------------------------- #
# The EM-algorithm is finished                                            #
# Check on the EM-monotonicity                                            #
# Plotting the values of the observed loglikelihood versus the iterations #
# ----------------------------------------------------------------------- #

if(loglikplot & k>1){

  par(mai=c(0.84,0.8,0.012,0.004))
  par(las = 3)
  par(cex.axis=0.7)
  par(cex.lab=1.2)
  plot(0:(iteration-2),loglik[1:(iteration-1)],type="l",axes = FALSE,xlab="iterations",ylab="log-likelihood",lwd=2)
  axis(1, at = 0:(iteration-2),label = 0:(iteration-2)) 
  axis(2)
  box(col = "black")

}

# --------------------- #
# Classification Matrix #
# --------------------- #

group <- apply(z,1,which.max)  

# --------------- #
# Standard Errors #
# --------------- #

StdErrors <- NULL
if(SE){
  
  if(familyX=="Gaussian"){
   
    if(familyY=="Gaussian"){
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> VarX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY 
        s[(3*k+2*k+1):(3*k+2*k+k)]               -> VarY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dnorm(X[i],mean=muX[j],sd=sqrt(VarX[j]),log=F)*dnorm(Y[i],mean=outer(X[i],0:1,"^")%*%betaY[j,],sd=sqrt(VarY[j]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,VarX,rbind(beta0,beta1),VarY[1,])
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEVarX  <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEVarY  <- se[(3*k+2*k+1):(3*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEVarX=SEVarX,SEcoef=SEcoef,SEVarY=SEVarY)
      
    }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> VarX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dnorm(X[i],mean=muX[j],sd=sqrt(VarX[j]),log=F)*dpois(Y[i],lambda=exp(outer(X[i],0:1,"^")%*%betaY[j,]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,VarX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEVarX  <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEVarX=SEVarX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k,mY=mY){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> VarX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dnorm(X[i],mean=muX[j],sd=sqrt(VarX[j]),log=F)*dbinom(Y[i],size=mY,prob=exp(outer(X[i],0:1,"^")%*%betaY[j,])/(1+exp(outer(X[i],0:1,"^")%*%betaY[j,])),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,VarX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,mY=mY,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEVarX  <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEVarX=SEVarX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> VarX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY
        s[(3*k+2*k+1):(3*k+2*k+k)]               -> nuY 
                
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dnorm(X[i],mean=muX[j],sd=sqrt(VarX[j]),log=F)*dgam(Y[i], mu = exp(outer(X[i],0:1,"^")%*%betaY[j,]), nu = nuY[j], log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,VarX,rbind(beta0,beta1),nuY)
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#      OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEVarX  <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEnuY   <- se[(3*k+2*k+1):(3*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEVarX=SEVarX,SEcoef=SEcoef,SEnuY=SEnuY)
      
    }
    
  }
  
  if(familyX=="Poisson"){
    
    if(familyY=="Gaussian"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        #s[(2*k+1):(3*k)]                         -> VarX
        #matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY 
        #s[(3*k+2*k+1):(3*k+2*k+k)]               -> VarY 
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY 
        s[(2*k+2*k+1):(2*k+2*k+k)]               -> VarY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dpois(X[i],lambda=muX[j],log=F)*dnorm(Y[i],mean=outer(X[i],0:1,"^")%*%betaY[j,],sd=sqrt(VarY[j]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,rbind(beta0,beta1),VarY[1,])
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEVarY  <- se[(2*k+2*k+1):(2*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef,SEVarY=SEVarY)
      
    }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dpois(X[i],lambda=muX[j],log=F)*dpois(Y[i],lambda=exp(outer(X[i],0:1,"^")%*%betaY[j,]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se        <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k,mY=mY){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dpois(X[i],lambda=muX[j],log=F)*dbinom(Y[i],size=mY,prob=exp(outer(X[i],0:1,"^")%*%betaY[j,])/(1+exp(outer(X[i],0:1,"^")%*%betaY[j,])),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,mY=mY,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se        <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY
        s[(2*k+2*k+1):(2*k+2*k+k)]               -> nuY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dpois(X[i],lambda=muX[j],log=F)*dgam(Y[i], mu = exp(outer(X[i],0:1,"^")%*%betaY[j,]), nu = nuY[j], log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,VarX,rbind(beta0,beta1),nuY)
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEnuY   <- se[(2*k+2*k+1):(2*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef,SEnuY=SEnuY)
      
    }
    
  }  
  
  if(familyX=="Binomial"){
    
    if(familyY=="Gaussian"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k,mX=mX){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY 
        s[(2*k+2*k+1):(2*k+2*k+k)]               -> VarY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dbinom(X[i],size=mX,prob=muX[j]/mX,log=F)*dnorm(Y[i],mean=outer(X[i],0:1,"^")%*%betaY[j,],sd=sqrt(VarY[j]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX/mX,rbind(beta0,beta1),VarY[1,])
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,mX=mX,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEVarY  <- se[(2*k+2*k+1):(2*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef,SEVarY=SEVarY)
      
    }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k,mX=mX){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dbinom(X[i],size=mX,prob=muX[j]/mX,log=F)*dpois(Y[i],lambda=exp(outer(X[i],0:1,"^")%*%betaY[j,]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX/mX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,mX=mX,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k,mX=mX,mY=mY){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dbinom(X[i],size=mX,prob=muX[j]/mX,log=F)*dbinom(Y[i],size=mY,prob=exp(outer(X[i],0:1,"^")%*%betaY[j,])/(1+exp(outer(X[i],0:1,"^")%*%betaY[j,])),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX/mX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,mX=mX,mY=mY,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se        <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k,mX=mX){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        matrix(s[(2*k+1):(2*k+2*k)],k,2,byrow=T) -> betaY
        s[(2*k+2*k+1):(2*k+2*k+k)]               -> nuY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dbinom(X[i],size=mX,prob=muX[j]/mX,log=F)*dgam(Y[i], mu = exp(outer(X[i],0:1,"^")%*%betaY[j,]), nu = nuY[j], log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX/mX,rbind(beta0,beta1),nuY)
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,mX=mX,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se        <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEcoef  <- matrix(se[(2*k+1):(2*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEnuY   <- se[(2*k+2*k+1):(2*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEcoef=SEcoef,SEnuY=SEnuY)
      
    }
    
  }
  
  if(familyX=="Gamma"){
    
    if(familyY=="Gaussian"){  
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> nuX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY 
        s[(3*k+2*k+1):(3*k+2*k+k)]               -> VarY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dgam(X[i],mu = muX[j],nu = nuX[j],log=F)*dnorm(Y[i],mean=outer(X[i],0:1,"^")%*%betaY[j,],sd=sqrt(VarY[j]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,nuX,rbind(beta0,beta1),VarY[1,])
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEnuX   <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEVarY  <- se[(3*k+2*k+1):(3*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEnuX=SEnuX,SEcoef=SEcoef,SEVarY=SEVarY)
      
    }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> nuX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dgam(X[i],mu = muX[j],nu = nuX[j],log=F)*dpois(Y[i],lambda=exp(outer(X[i],0:1,"^")%*%betaY[j,]),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,nuX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEnuX   <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEnuX=SEnuX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k,mY=mY){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> nuX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dgam(X[i],mu = muX[j],nu = nuX[j],log=F)*dbinom(Y[i],size=mY,prob=exp(outer(X[i],0:1,"^")%*%betaY[j,])/(1+exp(outer(X[i],0:1,"^")%*%betaY[j,])),log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,nuX,rbind(beta0,beta1))
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,mY=mY,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEnuX   <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEnuX=SEnuX,SEcoef=SEcoef)
      
    }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      fmin <- function(s,Y=Y,X=X,n=n,k=k){
        
        s[1:k]                                   -> weights
        s[(k+1):(2*k)]                           -> muX
        s[(2*k+1):(3*k)]                         -> nuX
        matrix(s[(3*k+1):(3*k+2*k)],k,2,byrow=T) -> betaY
        s[(3*k+2*k+1):(3*k+2*k+k)]               -> nuY 
        
        # Observed-data log-likelihood
        
        l  <- 0    
        for(i in 1:n){
          lj <- 0
          for(j in 1:k){
            lj <- lj + weights[j]*dgam(X[i],mu = muX[j],nu = nuX[j],log=F)*dgam(Y[i], mu = exp(outer(X[i],0:1,"^")%*%betaY[j,]), nu = nuY[j], log=F)
          }
          l <- l + log(lj)
        } 
        
        return(-l)
        
      }
      
      start.par <- c(prior,muX,nuX,rbind(beta0,beta1),nuY)
#       lower     <- numeric(npar+1)
#       upper     <- numeric(npar+1)
#       tol       <- 0.00000001
#       lower     <- start.par-tol
#       upper     <- start.par+tol
#       SE        <- optim(par=start.par,fn=fmin,hessian=TRUE,method="L-BFGS-B",lower=lower,upper=upper,Y=Y,X=X,n=n,k=k,control=list(maxit=100)) # fnscale=-1,
#       OI        <- solve(SE$hessian)
      Hes     <- hessian(func=fmin, x=start.par, method="Richardson",Y=Y,X=X,n=n,k=k)
      OI      <- solve(Hes)
      se      <- sqrt(diag(OI))
      SEprior <- se[1:k]
      SEmuX   <- se[(k+1):(2*k)]
      SEnuX   <- se[(2*k+1):(3*k)]
      SEcoef  <- matrix(se[(3*k+1):(3*k+2*k)],k,2,byrow=T)
      dimnames(SEcoef)[[1]] <- paste("comp",1:k,sep="")
      dimnames(SEcoef)[[2]] <- paste("beta",0:1,sep="")
      SEnuY   <- se[(3*k+2*k+1):(3*k+2*k+k)]
      
      StdErrors <- list(SEprior=SEprior,SEmuX=SEmuX,SEnuX=SEnuX,SEcoef=SEcoef,SEnuY=SEnuY)
      
    }
    
  }  
  
}

else{
  
  if(familyX=="Gaussian"){
    
    if(familyY=="Gaussian"){
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      }
    
  }
  
  if(familyX=="Poisson"){
    
    if(familyY=="Gaussian"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      }
    
  }  
  
  if(familyX=="Binomial"){
    
    if(familyY=="Gaussian"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + k + 3*k # for AIC, BIC and ICL
      
      }
    
  }
  
  if(familyX=="Gamma"){
    
    if(familyY=="Gaussian"){  
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Poisson"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Binomial"){
      
      npar <- (k-1) + 2*k + 2*k # for AIC, BIC and ICL
      
      }
    
    if(familyY=="Gamma"){
      
      npar <- (k-1) + 2*k + 3*k # for AIC, BIC and ICL
      
      }
    
  }  
  
}

###########################
## Information criteria  ##
###########################

AIC       <- 2*finalloglik - npar*2
BIC       <- 2*finalloglik - npar*log(n)

z.const    <- (z<10^(-322))*10^(-322)+(z>10^(-322))*z   # vincolo per evitare i NaN nel calcolo di tau*log(tau)
hard.z     <- (matrix(rep(apply(z,1,max),k),n,k,byrow=F)==z)*1
ECM        <- sum(hard.z*log(z.const))
ICL        <- BIC+ECM

beta <- cbind(beta0,beta1)
dimnames(beta)[[1]] <- paste("comp",1:k,sep="")
dimnames(beta)[[2]] <- paste("beta",0:1,sep="")

result <- list(
  Y         = Y,            # Y: a numerical vector for the response variable
  X         = X,            # X: a numerical vector for the unique covariate
  familyX   = familyX,      # the exponential distribution used for X 
  familyY   = familyY,      # the exponential distribution used for Y|x 
  k         = k,            # number of groups
  n         = n,            # sample size
  mX        = mX,           # mX=1 for Bernoulli; mX>1 for Binomial
  mY        = mY,           # mY=1 for Bernoulli; mY>1 for Binomial
  prior     = prior,
  muX       = muX,
  dispX     = dispX,
  VarFunX   = VarFunX,
  VarX      = VarX,
  nuX       = nuX,
  PX        = PX,
  beta      = beta,
  muY       = muY,
  dispY     = dispY,
  VarFunY   = VarFunY,
  VarY      = VarY,
  nuY       = nuY,
  PY        = PY,
  iter.stop = iteration,
  z         = z,
  group     = group,
  loglik    = finalloglik,
  AIC       = AIC,
  BIC       = BIC,
  ICL       = ICL,      # alla McNicholas
  SE        = StdErrors,
  call      = match.call()
)

class(result) <- "EECWM"
return(result)

alarm()

}

#################
## PLOT METHOD ##
#################

plot.EECWM <- function(x,plottype="CW",theta=30,nbin=35,contour=TRUE,nlevels=21,...)
{
  # theta is used only for plottype="density" and governs the orientation of the 3D-plot
  # nbin is used only for plottype="CW" and governs the number od equidistant bins
  # contour is not used for plottype="density"; if contour=TRUE, the contour lines are superimposed on the plot
  # nlevels is not used for plottype="density"; it governs the number of contour lines
  
  if(!any(plottype==c("density","scatter","CW"))){
    stop("error: the value specified for plottype is not density, scatter, or CW")}
  eval(parse(text=paste0(".",plottype,".plot.EECWM(x=x,theta=theta,nbin=nbin,contour=contour,nlevels=nlevels,...)")))
  #if(CI & any(plottype==c("obsfit","fitted")& !is.null(x$ex))){.ci.plot(x=x,...)}
}

.density.plot.EECWM <- function(x,theta=30,...)
{
  X       <- x$X
  Y       <- x$Y
  k       <- x$k
  familyX <- x$familyX
  familyY <- x$familyY
  prec    <- 101
  perc    <- 0.1
   
  if(familyX=="Gaussian"){
    
    if(familyY=="Gaussian"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){  	  	
          r <- x$prior[h]*dnorm(gridx,mean=x$muX[h],sd=sqrt(x$VarX[h]))*dnorm(gridy,mean=x$beta[h,1]+x$beta[h,2]*gridx,sd=sqrt(x$VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      )        
    
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dnorm(gridx,mean=x$muX[h],sd=sqrt(x$VarX[h]))*dpois(gridy,lambda=exp(x$beta[h,1]+x$beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridy){
        cont <- cont + 1
        lines(trans3d(gridx, y = q, z = fmisc[,cont], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridy){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(gridx, q, fcomp[,cont,h], pmat = res), col=h, lty=(h+1))
#       }
      
    }
    
    if(familyY=="Binomial"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dnorm(gridx,mean=x$muX[h],sd=sqrt(x$VarX[h]))*dbinom(gridy,size=x$mY,prob=exp(x$beta[h,1]+x$beta[h,2]*gridx)/(1+exp(x$beta[h,1]+x$beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridy){
        cont <- cont + 1
        lines(trans3d(gridx, y = q, z = fmisc[,cont], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridy){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(gridx, q, fcomp[,cont,h], pmat = res), col=h, lty=(h+1))
#       }
      
    }
    
    if(familyY=="Gamma"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dnorm(gridx,mean=x$muX[h],sd=sqrt(x$VarX[h]))*dgam(gridy,mu=exp(x$beta[h,1]+x$beta[h,2]*gridx),nu=x$nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      )        
      
    }
    
  }
  
  if(familyX=="Poisson"){
    
    if(familyY=="Gaussian"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dpois(gridx,lambda=x$muX[h])*dnorm(gridy,mean=x$beta[h,1]+x$beta[h,2]*gridx,sd=sqrt(x$VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridx){
        cont <- cont + 1
        lines(trans3d(q, gridy, z = fmisc[cont,], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridx){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(q, gridy, fcomp[cont,,h], pmat = res), col=h, lty=(h+1))
#       }
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dpois(gridx,lambda=x$muX[h])*dpois(gridy,lambda=exp(x$beta[h,1]+x$beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      contx <- 0
      for(q in gridx){
        contx <- contx + 1
        conty <- 0
        for(p in gridy){
          conty <- conty + 1
          xy0 <- trans3d(x = q,y = p,z = 0, pmat = res)
          xy1 <- trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res)
          segments(xy0$x,xy0$y,xy1$x,xy1$y)          
          points(trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res), col = "black", lwd=2, cex=0.6, pch=20)
        }
      }
      
    }
    
    if(familyY=="Binomial"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dpois(gridx,lambda=x$muX[h])*dbinom(gridy,size=x$mY,prob=exp(x$beta[h,1]+x$beta[h,2]*gridx)/(1+exp(x$beta[h,1]+x$beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      contx <- 0
      for(q in gridx){
        contx <- contx + 1
        conty <- 0
        for(p in gridy){
          conty <- conty + 1
          xy0 <- trans3d(x = q,y = p,z = 0, pmat = res)
          xy1 <- trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res)
          segments(xy0$x,xy0$y,xy1$x,xy1$y)          
          points(trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res), col = "black", lwd=2, cex=0.6, pch=20)
        }
      }
      
    }
    
    if(familyY=="Gamma"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dpois(gridx,lambda=x$muX[h])*dgam(gridy,mu=exp(x$beta[h,1]+x$beta[h,2]*gridx),nu=x$nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridx){
        cont <- cont + 1
        lines(trans3d(q, gridy, z = fmisc[cont,], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridx){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(q, gridy, fcomp[cont,,h], pmat = res), col=h, lty=(h+1))
#       }       
      
    }
    
  }
  
  if(familyX=="Binomial"){
    
    if(familyY=="Gaussian"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dbinom(gridx,size=x$mX,prob=x$muX[h]/x$mX)*dnorm(gridy,mean=x$beta[h,1]+x$beta[h,2]*gridx,sd=sqrt(x$VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridx){
        cont <- cont + 1
        lines(trans3d(q, gridy, z = fmisc[cont,], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridx){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(q, gridy, fcomp[cont,,h], pmat = res), col=h, lty=(h+1))
#       }
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dbinom(gridx,size=x$mX,prob=x$muX[h]/x$mX)*dpois(gridy,lambda=exp(x$beta[h,1]+x$beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      contx <- 0
      for(q in gridx){
        contx <- contx + 1
        conty <- 0
        for(p in gridy){
          conty <- conty + 1
          xy0 <- trans3d(x = q,y = p,z = 0, pmat = res)
          xy1 <- trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res)
          segments(xy0$x,xy0$y,xy1$x,xy1$y)          
          points(trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res), col = "black", lwd=2, cex=0.6, pch=20)
        }
      }
      
    }
    
    if(familyY=="Binomial"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dbinom(gridx,size=x$mX,prob=x$muX[h]/x$mX)*dbinom(gridy,size=x$mY,prob=exp(x$beta[h,1]+x$beta[h,2]*gridx)/(1+exp(x$beta[h,1]+x$beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      contx <- 0
      for(q in gridx){
        contx <- contx + 1
        conty <- 0
        for(p in gridy){
          conty <- conty + 1
          xy0 <- trans3d(x = q,y = p,z = 0, pmat = res)
          xy1 <- trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res)
          segments(xy0$x,xy0$y,xy1$x,xy1$y)          
          points(trans3d(x = q,y = p,z = fmisc[contx,conty], pmat = res), col = "black", lwd=2, cex=0.6, pch=20)
        }
      }
      
    }
    
    if(familyY=="Gamma"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dbinom(gridx,size=x$mX,prob=x$muX[h]/x$mX)*dgam(gridy,mu=exp(x$beta[h,1]+x$beta[h,2]*gridx),nu=x$nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridx){
        cont <- cont + 1
        lines(trans3d(q, gridy, z = fmisc[cont,], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridx){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(q, gridy, fcomp[cont,,h], pmat = res), col=h, lty=(h+1))
#       }       
      
    }
    
  }
  
  if(familyX=="Gamma"){
    
    if(familyY=="Gaussian"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dgam(gridx,mu=x$muX[h],nu=x$nuX[h])*dnorm(gridy,mean=x$beta[h,1]+x$beta[h,2]*gridx,sd=sqrt(x$VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      )        
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dgam(gridx,mu=x$muX[h],nu=x$nuX[h])*dpois(gridy,lambda=exp(x$beta[h,1]+x$beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridy){
        cont <- cont + 1
        lines(trans3d(gridx, y = q, z = fmisc[,cont], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridy){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(gridx, q, fcomp[,cont,h], pmat = res), col=h, lty=(h+1))
#       }
      
    }
    
    if(familyY=="Binomial"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dgam(gridx,mu=x$muX[h],nu=x$nuX[h])*dbinom(gridy,size=x$mY,prob=exp(x$beta[h,1]+x$beta[h,2]*gridx)/(1+exp(x$beta[h,1]+x$beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", 
            border = F,
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      ) -> res
      cont <- 0
      for(q in gridy){
        cont <- cont + 1
        lines(trans3d(gridx, y = q, z = fmisc[,cont], pmat = res), col = "black", lwd=2)
      }
#       cont <- 0
#       for(q in gridy){
#         cont <- cont + 1
#         for(h in 1:k)
#           lines(trans3d(gridx, q, fcomp[,cont,h], pmat = res), col=h, lty=(h+1))
#       }
      
    }
    
    if(familyY=="Gamma"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- x$prior[h]*dgam(gridx,mu=x$muX[h],nu=x$nuX[h])*dgam(gridy,mu=exp(x$beta[h,1]+x$beta[h,2]*gridx),nu=x$nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(cex.axis = 0.3)   
      par(cex.lab = 1.2)
      persp(x = gridx, y = gridy, z = fmisc,
            main="", #Joint Density from the Poisson LCWM
            xlab = "x",
            ylab = "y",
            zlab = "density", #density
            theta = theta, 
            axes = TRUE, 
            box = TRUE
      )        
      
    }
    
  }  
  
}

.CW.plot.EECWM <- function(x,nbin=35,contour=contour,nlevels=21,...)
{
  X       <- x$X
  Y       <- x$Y
  k       <- x$k
  familyX <- x$familyX
  familyY <- x$familyY
  prec    <- 101
  perc    <- 0.1
  group   <- x$group
  prior   <- x$prior
  beta    <- x$beta
  n       <- x$n
  
  if(familyX=="Gaussian"){
    
    muX     <- x$muX
    VarX    <- x$VarX
        
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
            
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){      	
          r <- prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) 
		    fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dnorm(z,mean=muX[h],sd=sqrt(VarX[h])) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dnorm(x,mean=muX[h],sd=sqrt(VarX[h])),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2)
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dnorm(z,mean=muX[h],sd=sqrt(VarX[h])) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dnorm(x,mean=muX[h],sd=sqrt(VarX[h])),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dnorm(z,mean=muX[h],sd=sqrt(VarX[h])) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dnorm(x,mean=muX[h],sd=sqrt(VarX[h])),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) 
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dnorm(z,mean=muX[h],sd=sqrt(VarX[h])) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dnorm(x,mean=muX[h],sd=sqrt(VarX[h])),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
  if(familyX=="Poisson"){
    
    muX     <- x$muX
    
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dpois(z,lambda=muX[h]) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dpois(xpoints,lambda=muX[h]),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dpois(z,lambda=muX[h]) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dpois(xpoints,lambda=muX[h]),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dpois(z,lambda=muX[h]) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dpois(xpoints,lambda=muX[h]),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dpois(z,lambda=muX[h]) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dpois(xpoints,lambda=muX[h]),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
  if(familyX=="Binomial"){
    
    muX     <- x$muX
    mX      <- x$mX
    
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dbinom(z,size=mX,prob=muX[h]/mX) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dbinom(xpoints,size=mX,prob=muX[h]/mX),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dbinom(z,size=mX,prob=muX[h]/mX) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dbinom(xpoints,size=mX,prob=muX[h]/mX),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dbinom(z,size=mX,prob=muX[h]/mX) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dbinom(xpoints,size=mX,prob=muX[h]/mX),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dbinom(z,size=mX,prob=muX[h]/mX) + iniz
        return(iniz)
      }
      
      xpoints <- seq(min(X),max(X),by=1) 
      fac     <- factor(X,levels=xpoints)
      ypoints <- table(fac)/n
      plot(xpoints,ypoints,main="",xlab="",type="h",lwd=3,ylab="probability",col="gray",axes=F,ylim=c(0,max(ypoints)+0.01))
      axis(1,at=xpoints,labels=xpoints)
      axis(2,at=c(0,max(ypoints)/2,max(ypoints)),labels=c(0,round(max(ypoints)/2,3),round(max(ypoints),3)))
      box()
      points(xpoints,mixfun(xpoints),lwd=2,pch=5)
      for(h in 1:k)
        points(xpoints,prior[h]*dbinom(xpoints,size=mX,prob=muX[h]/mX),col=h,pch=20) 
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
  if(familyX=="Gamma"){
    
    muX    <- x$muX
    nuX    <- x$nuX
    
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) 
        fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dgam(z,mu=muX[h],nu=nuX[h]) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dgam(x,mu=muX[h],nu=nuX[h]),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dgam(z,mu=muX[h],nu=nuX[h]) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dgam(x,mu=muX[h],nu=nuX[h]),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dgam(z,mu=muX[h],nu=nuX[h]) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dgam(x,mu=muX[h],nu=nuX[h]),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) 
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      # ----------------------------------
      # set up plotting area for histogram
      # ----------------------------------
      
      par(fig=c(0,1,0.65,1))
      par(cex.axis=0.6) 
      
      mixfun <- function(z){ #,pesi=x$prior,m=x$muX,s=sqrt(x$VarX)
        iniz <- 0
        for(h in 1:k)
          iniz <- prior[h]*dgam(z,mu=muX[h],nu=nuX[h]) + iniz
        return(iniz)
      }
      
      plot(hist(X,breaks=seq(min(X),max(X),length=nbin)),freq=F,main="",xlab="") #,breaks=seq(min(X),max(X),length=35) ,xlab=expression(italic(x))
      curve(mixfun(x),add=T,lwd=2,n=1001)
      for(h in 1:k)
        curve(prior[h]*dgam(x,mu=muX[h],nu=nuX[h]),add=T,lty=h,n=1001,col=h) 
      for(h in 1:k) 
        rug(X[which(group==h)],col=h)
      
      # ------------------------------------
      # set up plotting area for scatterplot
      # ------------------------------------
      
      par(fig=c(0,1,0,0.75),new=TRUE)
      
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
}

.scatter.plot.EECWM <- function(x,contour=contour,nlevels=21,...)
{
  X       <- x$X
  Y       <- x$Y
  k       <- x$k
  familyX <- x$familyX
  familyY <- x$familyY
  prec    <- 101
  perc    <- 0.1
  group   <- x$group
  prior   <- x$prior
  beta    <- x$beta
  n       <- x$n
  
  if(familyX=="Gaussian"){
    
    muX     <- x$muX
    VarX    <- x$VarX
    
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) 
        fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2)
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) 
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dnorm(gridx,mean=muX[h],sd=sqrt(VarX[h]))*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
  if(familyX=="Poisson"){
    
    muX     <- x$muX
    
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dpois(gridx,lambda=muX[h])*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
  if(familyX=="Binomial"){
    
    muX     <- x$muX
    mX      <- x$mX
    
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(lx,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq(min(X),max(X),by=1)
      lx    <- length(gridx)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(lx,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dbinom(gridx,size=mX,prob=muX[h]/mX)*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,lx,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
  if(familyX=="Gamma"){
    
    muX    <- x$muX
    nuX    <- x$nuX
    
    if(familyY=="Gaussian"){
      
      VarY    <- x$VarY 
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dnorm(gridy,mean=beta[h,1]+beta[h,2]*gridx,sd=sqrt(VarY[1,h]))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) 
        fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,beta[h,1]+beta[h,2]*griglia,lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Poisson"){
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- x$prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dpois(gridy,lambda=exp(beta[h,1]+beta[h,2]*gridx))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Binomial"){
      
      mY <- x$mY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq(min(Y),max(Y),by=1)
      ly    <- length(gridy)
      
      fcomp <- array(0,c(prec,ly,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dbinom(gridy,size=mY,prob=exp(beta[h,1]+beta[h,2]*gridx)/(1+exp(beta[h,1]+beta[h,2]*gridx)))
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,ly)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) 
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,mY*exp(beta[h,1]+beta[h,2]*griglia)/(1+exp(beta[h,1]+beta[h,2]*griglia)),lty=h,col=h,type="l")
      }      
      
    }
    
    if(familyY=="Gamma"){
      
      nuY <- x$nuY
      
      gridx <- seq((min(X)-perc*(max(X)-min(X))),(max(X)+perc*(max(X)-min(X))),length=prec)
      gridy <- seq((min(Y)-perc*(max(Y)-min(Y))),(max(Y)+perc*(max(Y)-min(Y))),length=prec)
      
      fcomp <- array(0,c(prec,prec,k))
      for(h in 1:k){
        f <- function(gridx,gridy){        
          r <- prior[h]*dgam(gridx,mu=muX[h],nu=nuX[h])*dgam(gridy,mu=exp(beta[h,1]+beta[h,2]*gridx),nu=nuY[h])
        }
        fcomp[,,h] <- outer(gridx,gridy,f) 
      }
      fmisc <- matrix(0,prec,prec)
      for(h in 1:k) fmisc <- fmisc + fcomp[,,h]
      
      par(mai=c(1.1,1.1,0.01,0.01))
      plot(X,Y,col=group,cex=0.6,pch=group,lwd=2) 
      if(contour==TRUE)
        contour(x = gridx, y = gridy, z = fmisc, nlevels = nlevels, col="lightgray", drawlabels=TRUE, add=T) #
      points(X, Y, cex=0.6, pch=group, col=group)
      for(h in 1:k){ 
        griglia <- seq(min(X[which(group==h)]),max(X[which(group==h)]),length=101)
        points(griglia,exp(beta[h,1]+beta[h,2]*griglia),lty=h,col=h,type="l")
      }      
      
    }
    
  }
  
}

