######################
## Gaussian case: X ## 
######################

GaussianX <- function(X,weights){ 
  
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  mu.obs    <- weighted.mean(x=X,w=weights)
  sigma.obs <- sqrt(sum(weights*(X-mu.obs)^2)/sum(weights))
  l         <- sum(weights*dnorm(x=X, mean = mu.obs, sd = sigma.obs, log = TRUE))
    
  return(
    list(
      loglik = l,
      muX    = mu.obs,
      sigmaX = sigma.obs
    )
  )  
  
}

########################
## Gaussian case: Y|x ## 
########################

GaussianY <- function(Y,X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY      <- glm(Y ~ X,family=gaussian(),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs    <- modelY$coefficients
    sigma.obs   <- sqrt(summary(modelY)$dispersion)
    
    starting.values <- c(beta.obs,log(sigma.obs))
    
  }
  
  f <- function(par,Y,X,weights){
    
    beta0 <- par[1]
    beta1 <- par[2]
    sigma <- exp(par[3])
    
    l <- -sum(weights*dnorm(x=Y, mean = beta0+beta1*X, sd = sigma, log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y,X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est[1]
  beta1.hat <- est[2]
  sigma.hat <- exp(est[3])
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat,
      beta1  = beta1.hat,
      sigmaY = sigma.hat
    )
  )  
  
}

####################################
## Gaussian case: Y|x for the LCM ## 
####################################

GaussianY2 <- function(Y,X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY      <- glm(Y ~ 1,family=gaussian(),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs    <- modelY$coefficients
    sigma.obs   <- sqrt(summary(modelY)$dispersion)
    
    starting.values <- c(beta.obs,log(sigma.obs))
    
  }
  
  f <- function(par,Y,weights){
    
    beta0 <- par[1]
    sigma <- exp(par[2])
    
    l <- -sum(weights*dnorm(x=Y, mean = beta0, sd = sigma, log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est[1]
  sigma.hat <- exp(est[2])
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat,
      sigmaY = sigma.hat
    )
  )  
  
}

#####################
## Poisson case: X ##
#####################

PoissonX <- function(X,weights){ #,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1
  
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  lambda.obs  <- weighted.mean(x=X,w=weights)
  
  f <- function(par,X,weights){
    
    lambda <- par
    
    l <- -sum(weights*dpois(x=X, lambda = lambda, log = TRUE))
    
  }
  
  res <- optimize(f=f, interval=c(0,2*lambda.obs), X=X, weights=weights)
    
  loglik     <- -res$objective
  lambda.hat<-  res$minimum
  
  return(
    list(
      loglik  = loglik,
      lambdaX = lambda.hat
    )
  )  
  
}

#######################
## Poisson case: Y|x ## 
#######################

PoissonY <- function(Y,X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY      <- glm(Y ~ X,family=poisson(),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs    <- modelY$coefficients
    
    starting.values <- c(beta.obs)
    
  }
  
  f <- function(par,Y,X,weights){
    
    beta0 <- par[1]
    beta1 <- par[2]
    
    l <- -sum(weights*dpois(x=Y, lambda = exp(beta0+beta1*X), log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est[1]
  beta1.hat <- est[2]
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat,
      beta1  = beta1.hat
    )
  )  
  
}

###################################
## Poisson case: Y|x for the LCM ## 
###################################

PoissonY2 <- function(Y,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY      <- glm(Y ~ 1,family=poisson(),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs    <- modelY$coefficients
    
    starting.values <- beta.obs
    
  }
  
  f <- function(par,Y,weights){
    
    beta0 <- par
    
    l <- -sum(weights*dpois(x=Y, lambda = exp(beta0), log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat
    )
  )  
  
}


######################
## Binomial case: X ##
######################

BinomialX <- function(X,weights,m){ #,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1
  
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  # p.obs  <- weighted.mean(x=X/m,w=weights)
  
  f <- function(par,X,weights,m){
    
    p <- par
    
    l <- -sum(weights*dbinom(x=X, size = m, prob=p, log = TRUE))
    
  }
  
  res <- optimize(f=f, interval=c(0,1), X=X, weights=weights, m=m)
  
  loglik <- -res$objective
  p.hat  <-  res$minimum
  
  return(
    list(
      loglik = loglik,
      pX     = p.hat
    )
  )  
  
}

########################
## Binomial case: Y|x ## 
########################

BinomialY <- function(Y,X,weights,m,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
    
  if(start==FALSE){
    
    modelY   <- glm(cbind(Y,m-Y) ~ X,family=binomial(),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs <- modelY$coefficients
    
    starting.values <- c(beta.obs)
    
  }
  
  f <- function(par,Y,X,weights,m){
    
    beta0 <- par[1]
    beta1 <- par[2]
    
    l <- -sum(weights*dbinom(x=Y, size = m, p = exp(beta0+beta1*X)/(1+exp(beta0+beta1*X)), log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, X=X, weights=weights, m=m, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, X=X, weights=weights, m=m, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est[1]
  beta1.hat <- est[2]
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat,
      beta1  = beta1.hat
    )
  )  
  
}

####################################
## Binomial case: Y|x for the LCM ## 
####################################

BinomialY2 <- function(Y,weights,m,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY   <- glm(cbind(Y,m-Y) ~ 1,family=binomial(),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs <- modelY$coefficients
    
    starting.values <- beta.obs
    
  }
  
  f <- function(par,Y,weights,m){
    
    beta0 <- par
    
    l <- -sum(weights*dbinom(x=Y, size = m, p = exp(beta0)/(1+exp(beta0)), log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, weights=weights, m=m, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, weights=weights, m=m, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat
    )
  )  
  
}

####################################
## re-parameterized Gamma density ##
####################################

dgam <- function(x,mu,nu,log=FALSE){
  
  dgamma(x, shape=nu, scale = mu/nu, log = log)
  
}

rgam <- function(x,mu,nu){
  
  rgamma(x, shape=nu, scale = mu/nu)
  
}

###################
## Gamma case: X ## 
###################

GammaX <- function(X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    mu.obs  <- weighted.mean(x=X,w=weights)
    var.obs <- sum(weights*(X-mu.obs)^2)/sum(weights)
    nu.obs  <- mu.obs^2/var.obs
    
    starting.values <- c(log(mu.obs),log(nu.obs))
    
  }
  
  f <- function(par,X,weights){
    
    mu <- exp(par[1])
    nu <- exp(par[2])
    
    l <- -sum(weights*dgam(x=X, mu = mu, nu = nu, log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  mu.hat <- exp(est[1])
  nu.hat <- exp(est[2])
  
  return(
    list(
      loglik = loglik,
      muX    = mu.hat,
      nuX    = nu.hat
    )
  )  
  
}

#####################
## Gamma case: Y|x ## 
#####################

GammaY <- function(Y,X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY    <- glm(Y ~ X,family=Gamma(link="log"),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs  <- modelY$coefficients
    nu.obs    <- 1/summary(modelY)$dispersion
    
    starting.values <- c(beta.obs,log(nu.obs))
    
  }
  
  f <- function(par,Y,X,weights){
    
    beta0 <- par[1]
    beta1 <- par[2]
    #mu    <- exp(beta0+beta1*X)
    nu    <- exp(par[3])
    
    #l <- 0
    #for(i in 1:length(X)){
      
      #l <- l + weights[i]*ifelse(mu[i]>0,dgam(x=Y[i], mu = mu[i], nu = nu, log = TRUE),-Inf)
      
    #}
    #l <- -l
    
    l <- -sum(weights*dgam(x=Y, mu = exp(beta0+beta1*X), nu = nu, log = TRUE))
    #print(l)
    #print(nu)
    #print(exp(beta0+beta1*X))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est[1]
  beta1.hat <- est[2]
  nu.hat    <- exp(est[3])
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat,
      beta1  = beta1.hat,
      nuY    = nu.hat
    )
  )  
  
}

#################################
## Gamma case: Y|x for the LCM ## 
#################################

GammaY2 <- function(Y,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY    <- glm(Y ~ 1,family=Gamma(link="log"),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
    beta.obs  <- modelY$coefficients
    nu.obs    <- 1/summary(modelY)$dispersion
    
    starting.values <- c(beta.obs,log(nu.obs))
    
  }
  
  f <- function(par,Y,weights){
    
    beta0 <- par[1]
    nu    <- exp(par[2])
    
    l <- -sum(weights*dgam(x=Y, mu = exp(beta0), nu = nu, log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta0.hat <- est[1]
  nu.hat    <- exp(est[2])
  
  return(
    list(
      loglik = loglik,
      beta0  = beta0.hat,
      nuY    = nu.hat
    )
  )  
  
}


############################################################################################################################
















# ####################################
# ## re-parameterized Gamma density ##
# ####################################
# 
# dinvnorm <- function(x,m,s,log=FALSE){
#   
#   library(statmod)
#   
#   dinvgauss(x, mu=m, lambda = 1/s, log = log)
#   
# }
# 
# ##############################
# ## Inverse Gaussian case: X ## 
# ##############################
# 
# InvGaussianX <- function(X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
#   
#   # X: a numerical vector for the unique covariate
#   # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
#   
#   library(rmutil)
#   
#   if(start==FALSE){
#     
#     mu.obs  <- weighted.mean(x=X,w=weights)
#     var.obs <- sum(weights*(X-mu.obs)^2)/sum(weights)
#     s.obs   <- var.obs/mu.obs^3
#     
#     starting.values <- c(log(mu.obs),log(s.obs))
#     
#   }
#   
#   f <- function(par,X,weights){
#     
#     mu <- exp(par[1])   
#     s  <- exp(par[2])
#     
#     l <- -sum(weights*dinvgauss(y=X, m = mu, s = s, log = TRUE))
#     
#   }
#   
#   if(start==FALSE)
#     res <- optim(par=starting.values, fn=f, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
#   if(start==TRUE)
#     res <- optim(par=initial.values, fn=f, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
#   
#   loglik  <- -res$value
#   est     <-  res$par
#   
#   mu.hat <- exp(est[1])
#   s.hat  <- exp(est[2])
#   
#   return(
#     list(
#       loglik = loglik,
#       muX    = mu.hat,
#       sX     = s.hat
#     )
#   )  
#   
# }
# 
# ################################
# ## Inverse Gaussian case: Y|x ## 
# ################################
# 
# InvGaussianY <- function(Y,X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
#   
#   # Y: a numerical vector for the response variable
#   # X: a numerical vector for the unique covariate
#   # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
#   
#   #library(rmutil)
#   library(statmod)
#   
#   if(start==FALSE){
#     
#     modelY    <- glm(Y ~ X,family=inverse.gaussian(),weights=weights,control=list(trace=TRUE,epsilon=1e-14)) 
#     beta.obs  <- modelY$coefficients
#     s.obs     <- summary(modelY)$dispersion
#     
#     starting.values <- c(beta.obs,log(s.obs))
#     print(starting.values)
#   }
#   
#   f <- function(par,Y,X,weights){
#     
#     beta0 <- par[1]
#     beta1 <- par[2]
#     #mu    <- exp(beta0+beta1*X)
#     s     <- exp(par[3])
#     
#     #l <- 0
#     #for(i in 1:length(X)){
#     
#     #l <- l + weights[i]*ifelse(mu[i]>0,dgam(x=Y[i], mu = mu[i], nu = nu, log = TRUE),-Inf)
#     
#     #}
#     #l <- -l
# 
#     #print(dinvgauss(y=Y, m = exp(beta0+beta1*X), s = s, log = TRUE))
#     
#     #l <- -sum(weights*dinvgauss(x=Y, mu = exp(beta0+beta1*X), lambda = s, log = TRUE))
#     #l <- -sum(weights*dinvgauss(y=Y, m = exp(beta0+beta1*X), s = s, log = TRUE))
#     l <- -sum(weights*dinvnorm(x=Y, m = exp(beta0+beta1*X), s = s, log = TRUE))
#     #print(l)
#     #print(nu)
#     
#     
#   }
#   
#   if(start==FALSE)
#     res <- optim(par=starting.values, fn=f, Y=Y, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
#   if(start==TRUE)
#     res <- optim(par=initial.values, fn=f, Y=Y, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
#   
#   loglik  <- -res$value
#   est     <-  res$par
#   
#   beta0.hat <- est[1]
#   beta1.hat <- est[2]
#   s.hat     <- exp(est[3])
#   
#   return(
#     list(
#       loglik = loglik,
#       beta0  = beta0.hat,
#       beta1  = beta1.hat,
#       sY     = s.hat
#     )
#   )  
#   
# }






























# ######################
# ## Gaussian case: X ## vanno gi bene mu.obs e sigma.obs
# ######################
# 
# GaussianXold <- function(X,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=1){ 
#   
#   # X: a numerical vector for the unique covariate
#   # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
#   
#   if(start==FALSE){
#     
#     mu.obs    <- weighted.mean(x=X,w=weights)
#     sigma.obs <- sqrt(sum(weights*(X-mu.obs)^2)/sum(weights))
#     
#     starting.values <- c(mu.obs,exp(sigma.obs))
#     
#   }
#  
#   f <- function(par,X,weights){
#     
#   mu    <- par[1]
#   sigma <- exp(par[2])
#   
#   l <- -sum(weights*dnorm(x=X, mean = mu, sd = sigma, log = TRUE))
#   
#   }
#   
#   if(start==FALSE)
#     res <- optim(par=starting.values, fn=f, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
#   if(start==TRUE)
#     res <- optim(par=initial.values, fn=f, X=X, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
#   
#   loglik  <- -res$value
#   est     <-  res$par
#   
#   mu.hat    <- est[1]
#   sigma.hat <- exp(est[2])
#   
#   return(
#     list(
#       loglik = loglik,
#       muX    = mu.hat,
#       sigmaX = sigma.hat
#     )
#   )  
#   
# }

# n    <- 500
# data <- rnorm(n,4,2)
# w    <- rbinom(n, size=1, prob=0.5) # rep(1,n)
# pippo <- GaussianX(data,weights=w)
# pippo


# n        <- 500
# X        <- rnorm(n, 4,2)
# weights  <- rbinom(n, size=1, prob=0.5) # rep(1,n)
# s        <- 2
# b0       <- 1
# b1       <- 1
# Y        <- rnorm(n,b0+b1*X,sd=s)
# pippo    <- GaussianY(Y,X,weights=weights)
# pippo


